##Gimp Python plugin - Simple Button 1
##Copyright (C) 2010 Andrei Roslovtsev ~ www.byes-and-pixels.com
##
##This program is free software; you can redistribute it and/or
##modify it under the terms of the GNU General Public License
##as published by the Free Software Foundation version 2
##of the License.
##
##This program is distributed in the hope that it will be useful,
##but WITHOUT ANY WARRANTY; without even the implied warranty of
##MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##GNU General Public License for more details.
##
##You should have received a copy of the GNU General Public License
##along with this program; if not, write to the Free Software
##Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
#
#
# Creates a simple web button.
# Accepted parameters are dimensions, color,
# corner radius, Add optional text - size, font, color etc.
#
# Works best with bright colors (lime?) and high saturation.
#
# Note:
# Developed and tested with Python 2.6 only.
#
# Version:1.0
# 2010-Dec-10
# Andrei Roslovtsev
# www.bytes-and-pixels.com
# andrei@bytes-and-pixels.com

#-----------------------------------------------------------


from gimpfu import *
import sys

def make_button(width_var,height_var,rounding,
                main_col,text_flag,
                text_size,button_txt,font,text_col,
                flatten_flag,crop_flag):

    if font==None:
        font='Sans'
        
    margin = 10
    
    # set image dimensions
    img_w = width_var+margin*2
    img_h = height_var+margin*2
        
    image = pdb.gimp_image_new(img_w, img_h, 0)
    pdb.gimp_image_set_filename(image, "Simple_button")

    # reset colors to B&W
    pdb.gimp_context_set_default_colors()
    
    layer1 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'BG', 100, 0)
    layer2 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Fill', 100, 0)
    pdb.gimp_image_add_layer(image, layer1, 0)
    pdb.gimp_image_add_layer(image, layer2,-1)
    
    display = pdb.gimp_display_new(image)
    gimp.pdb.gimp_edit_fill(layer1, 3)


# Edit Fill layer -------------------------------------

    pdb.gimp_round_rect_select(image, margin, margin, width_var,
                               height_var, rounding, rounding,
                               2, 1, 0, 1, 1)
    
    pdb.gimp_context_set_foreground(main_col)
    pdb.gimp_edit_bucket_fill(layer2, 0, 0, 100, 0, 0, 0, 0)

    #Save selection 1
    sel_1 = pdb.gimp_selection_save(image)
    
# Create shadow layer -------------------------------------

    layer3 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Shadow', 30, 0)
    pdb.gimp_image_add_layer(image, layer3, 0)
    
    pdb.gimp_selection_shrink(image, 2)
    pdb.gimp_selection_translate(image, 0, -2)

    #Save selection 2
    sel_2 = pdb.gimp_selection_save(image)

    pdb.gimp_selection_load(sel_1)
    pdb.gimp_selection_combine(sel_2, 1)

    # reset colors to B&W
    pdb.gimp_context_set_default_colors()

    pdb.gimp_edit_bucket_fill(layer3, 0, 0, 100, 0, 0, 0, 0)

    pdb.gimp_selection_load(sel_1)
    pdb.plug_in_blur(image, layer3)
    pdb.gimp_selection_clear(image)


# Create Highlight layer -------------------------------------

    layer4 = pdb.gimp_layer_new(image, img_w, img_h, 1, 'Highlight', 100, 5)
    pdb.gimp_image_add_layer(image, layer4, 0)

    pdb.gimp_selection_load(sel_2)
    pdb.gimp_rect_select(image, 0, img_h/2, img_w, img_h, 1, 0, 0)
    
    pdb.gimp_selection_translate(image, 0, 1)
    
    pdb.gimp_context_swap_colors()

    pdb.gimp_edit_blend(layer4, 2, 0, 0, 100, 0, 0,
                        0, 0, 0, 0, 1, img_w/2, margin,
                        img_w/2, img_h/2)
    
    pdb.gimp_selection_load(sel_1)
    pdb.plug_in_blur(image, layer4)
    pdb.gimp_selection_clear(image)


# Text Layer (optional) --------------------------------------------
    if text_flag == 1:
        layer7 = pdb.gimp_text_layer_new(image, button_txt, font, text_size, 0)
        pdb.gimp_image_add_layer(image, layer7, 0)
        pdb.gimp_text_layer_set_color(layer7, text_col)
        
        txt_x = (image.width-layer7.width)/2
        txt_y = (image.height-layer7.height)/2
        layer7.set_offsets(txt_x, txt_y-2)



# Flatten Image routine ---------------------------------------------
    if flatten_flag == 1:
        pdb.gimp_image_remove_layer(image, layer1)
        layer1 = pdb.gimp_image_merge_visible_layers(image, 0)

# Autocrop Image routine ---------------------------------------------
    if crop_flag == 1:
        if flatten_flag==0:
            pdb.gimp_message('Autocrop option only works with flattened image.')
            
        pdb.plug_in_autocrop(image, layer1)

    pdb.gimp_image_set_active_layer(image, layer1)
    
    # reset colors to B&W
    pdb.gimp_context_set_default_colors()
#--------------------------------------------------------------------
#Helper functions
#Author:  Joao S. O. Bueno
#source:  http://www.gimpusers.com/forums/gimp-user/
#    11870-set-active-layer-by-referring-to-its-name

def find_layer_by_name (image, name):
    for layer in image.layers:
        if layer.name == name:
            return layer
            return None

def find_gradient_by_name (name):
    num_gradients, gradient_list = pdb.gimp_gradients_get_list("")

    if name in gradient_list:
        return True
    return False


#---------------------------------------------------------------------
register(
    "Simple_Button_01",
    N_("Simple_Button_01"),
    "Create a web button, often seen as [Download] button",
    "Anadrei Roslovtsev",
    "www.bytes-and-pixels.com",
    "2010",
    N_("_Simple Web button 01..."),
    #"RGB*, GRAY*",
    "",
    [
        (PF_SPINNER, "width_var", "Button Width:", 140, (1, 2000, 1)),
        (PF_SPINNER, "height_var", "Button Height:", 40, (1, 2000, 1)),
        (PF_SPINNER, "rounding", "Corner Radius:", 20, (0, 100, 1)),
        (PF_COLOR, "main_col", "Color:", (35, 167, 66) ),
        (PF_TOGGLE, "text_flag", "Add Text:", 1),
        (PF_SPINNER, "text_size", "Text Size (px):", 18, (1, 2000, 1)),
        (PF_STRING, "button_txt", "Text:", "Download"),
        (PF_FONT, "font", "FONT:", 0),
        (PF_COLOR, "text_col", "Text Color:", (255, 255, 255) ),
        (PF_TOGGLE, "flatten_flag", "Flatten Image", 0),
        (PF_TOGGLE, "crop_flag", "Autocrop Result", 0)
    ],
    [],
    make_button,
    menu="<Image>/File/Create/Buttons/B&P",
    domain=("gimp20-python", gimp.locale_directory)
    )

main()
